from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Prompt")


@_attrs_define
class Prompt:
    """
    Attributes:
        prompt (str):
        id (Union[Unset, str]):
        version (Union[Unset, int]):
        root (Union[None, Unset, str]):
        parent (Union[None, Unset, str]):
        project (Union[None, Unset, str]):
    """

    prompt: str
    id: Union[Unset, str] = UNSET
    version: Union[Unset, int] = UNSET
    root: Union[None, Unset, str] = UNSET
    parent: Union[None, Unset, str] = UNSET
    project: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        prompt = self.prompt

        id = self.id

        version = self.version

        root: Union[None, Unset, str]
        if isinstance(self.root, Unset):
            root = UNSET
        else:
            root = self.root

        parent: Union[None, Unset, str]
        if isinstance(self.parent, Unset):
            parent = UNSET
        else:
            parent = self.parent

        project: Union[None, Unset, str]
        if isinstance(self.project, Unset):
            project = UNSET
        else:
            project = self.project

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "prompt": prompt,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if version is not UNSET:
            field_dict["version"] = version
        if root is not UNSET:
            field_dict["root"] = root
        if parent is not UNSET:
            field_dict["parent"] = parent
        if project is not UNSET:
            field_dict["project"] = project

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        prompt = d.pop("prompt")

        id = d.pop("id", UNSET)

        version = d.pop("version", UNSET)

        def _parse_root(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        root = _parse_root(d.pop("root", UNSET))

        def _parse_parent(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        parent = _parse_parent(d.pop("parent", UNSET))

        def _parse_project(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        project = _parse_project(d.pop("project", UNSET))

        prompt = cls(
            prompt=prompt,
            id=id,
            version=version,
            root=root,
            parent=parent,
            project=project,
        )

        prompt.additional_properties = d
        return prompt

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import typer
from cli.xleap.sdk import Xleap

from xleap.models import DataPoint

a = DataPoint(
    question="Question: What were the temperatures and snowfall amounts during the cold snap in Afghanistan in January 2023, and how many people and livestock were affected?",
    answer="During the cold snap in Afghanistan in January 2023, temperatures dropped to record lows, reaching as low as -30 degrees Celsius (-22 degrees Fahrenheit) in some regions. The snowfall amounts varied across the country, with some areas experiencing heavy snowfall of up to 2 meters (6.5 feet), while others received lighter snowfall of around 30 centimeters (1 foot).\n\nAs for the number of people and livestock affected, it is estimated that approximately 500,000 people and 1 million livestock were affected by the extreme cold and heavy snowfall. These severe weather conditions caused disruptions in transportation, power outages, and limited access to essential services, leading to significant challenges for the affected population.",
)
b = DataPoint(
    question="did any russian player play?",
    answer="Yes, Russian players participated in Wimbledon 2023.",
    contexts=[
        "players, after they were banned from the previous edition due to the Russian invasion of Ukraine.",
        "Mate Pavić /  Lyudmyla Kichenok def.  Joran Vliegen /  Xu Yifan, 6-4, 6-7(9-11), 6-3",
        "disrupted by rain.The tournament saw the return of Russian and Belarusian tennis players, after",
        "The tournament was played on grass courts, with all main draw matches played at the All England",
    ],
)


def main(name: str = "Xleap"):
    print(f"Hello {name}")

    client = Xleap()

    r = client.data.create(body=b)
    print(r.parsed)


if __name__ == "__main__":
    typer.run(main)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.run import Run


T = TypeVar("T", bound="ListRunsResponse200")


@_attrs_define
class ListRunsResponse200:
    """
    Attributes:
        next_ (Union[None, Unset, str]):
        previous (Union[None, Unset, str]):
        results (Union[Unset, List['Run']]):
    """

    next_: Union[None, Unset, str] = UNSET
    previous: Union[None, Unset, str] = UNSET
    results: Union[Unset, List["Run"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        next_: Union[None, Unset, str]
        if isinstance(self.next_, Unset):
            next_ = UNSET
        else:
            next_ = self.next_

        previous: Union[None, Unset, str]
        if isinstance(self.previous, Unset):
            previous = UNSET
        else:
            previous = self.previous

        results: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.results, Unset):
            results = []
            for results_item_data in self.results:
                results_item = results_item_data.to_dict()
                results.append(results_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if next_ is not UNSET:
            field_dict["next"] = next_
        if previous is not UNSET:
            field_dict["previous"] = previous
        if results is not UNSET:
            field_dict["results"] = results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.run import Run

        d = src_dict.copy()

        def _parse_next_(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        next_ = _parse_next_(d.pop("next", UNSET))

        def _parse_previous(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        previous = _parse_previous(d.pop("previous", UNSET))

        results = []
        _results = d.pop("results", UNSET)
        for results_item_data in _results or []:
            results_item = Run.from_dict(results_item_data)

            results.append(results_item)

        list_runs_response_200 = cls(
            next_=next_,
            previous=previous,
            results=results,
        )

        list_runs_response_200.additional_properties = d
        return list_runs_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import functools
import os
import uuid
import weakref
from typing import Optional

import requests
from langsmith import client
from requests import adapters as requests_adapters
from urllib3.util import Retry

from xleap.config import Keys


class L(client.Client):
    def __init__(
        self,
        api_url: Optional[str] = None,
        *,
        api_key: Optional[str] = None,
        retry_config: Optional[Retry] = None,
        timeout_ms: Optional[int] = None,
        web_url: Optional[str] = None,
        session: Optional[requests.Session] = None,
    ) -> None:
        """Initialize a Client instance.

        Parameters
        ----------
        api_url : str or None, default=None
            URL for the XLeap API. Defaults to the XLEAP_ENDPOINT
            environment variable or http://localhost:1984 if not set.
        api_key : str or None, default=None
            API key for the XLeap API. Defaults to the XLEAP_API_KEY
            environment variable.
        retry_config : Retry or None, default=None
            Retry configuration for the HTTPAdapter.
        timeout_ms : int or None, default=None
            Timeout in milliseconds for the HTTPAdapter.
        web_url : str or None, default=None
            URL for the XLeap web app. Default is auto-inferred from
            the ENDPOINT.
        session: requests.Session or None, default=None
            The session to use for requests. If None, a new session will be
            created.

        Raises
        ------
        LangSmithUserError
            If the API key is not provided when using the hosted service.
        """
        self.api_key = api_key or os.getenv(Keys.API_KEY.value)
        self.api_url = api_url or os.getenv(Keys.API_ENDPOINT.value)

        self.retry_config = retry_config or client._default_retry_config()
        self.timeout_ms = timeout_ms or 7000
        self._web_url = web_url
        self._tenant_id: Optional[uuid.UUID] = None
        # Create a session and register a finalizer to close it
        self.session = session if session else requests.Session()
        weakref.finalize(self, client.close_session, self.session)

        # Mount the HTTPAdapter with the retry configuration
        adapter = requests_adapters.HTTPAdapter(max_retries=self.retry_config)
        self.session.mount("http://", adapter)
        self.session.mount("https://", adapter)
        self._get_data_type_cached = functools.lru_cache(maxsize=10)(
            self._get_data_type
        )



import base64
import enum
import json
import logging
import os
import pathlib
from dataclasses import dataclass
from typing import Optional

XLEAP_CREDENTIAL_FILENAME = ".config/xleap-config.json"


logger = logging.getLogger(__name__)


class Keys(enum.StrEnum):
    USERNAME = "XLEAP_USERNAME"
    API_KEY = "XLEAP_API_KEY"
    PROJECT_ID = "XLEAP_PROJECT_ID"

    @property
    def suffix(self):
        return self.value[6:].lower()


@dataclass
class Auth:
    username: Optional[str] = None
    api_key: Optional[str] = None
    project_id: Optional[str] = None

    secrets_file = pathlib.Path.home() / pathlib.Path(XLEAP_CREDENTIAL_FILENAME)

    @property
    def auth_header(self) -> Optional[str]:
        if not self.username or not self.api_key:
            raise AttributeError("Failed: no authentication header available")

        token = f"{self.username}:{self.api_key}"
        return f"Basic {base64.b64encode(token.encode('ascii')).decode('ascii')}"

    def from_env(self) -> bool:
        """Load credentials from env variable"""
        logger.info("loading credentials from environment variables")

        for key in Keys:
            print(key.suffix, key.value)
            setattr(self, key.suffix, os.getenv(key.value))

        if not self.username:
            logger.warn("username is required")

        if not self.api_key:
            logger.warn("api_key is required")

        if not self.project_id:
            logger.warn("project_id is required")

        return self.auth_header

    def from_file(self) -> bool:
        """Load credentials from disk"""

        if not self.secrets_file.exists():
            logger.info("Credentials file not found.")
            return False

        with self.secrets_file.open() as creds:
            credentials = json.load(creds)
            for key in Keys:
                setattr(self, key.suffix, credentials.get(key.suffix, None))
            return True

    def save(self) -> None:
        """Save credentials to disk."""

        logger.debug("saving credentials")

        self.secrets_file.parent.mkdir(exist_ok=True, parents=True)

        with self.secrets_file.open("w") as creds:
            json.dump(
                {
                    f"{Keys.USERNAME.suffix}": self.username,
                    f"{Keys.API_KEY.suffix}": self.api_key,
                    f"{Keys.PROJECT_ID.suffix}": self.project_id,
                },
                creds,
            )

        logger.info("credentials saved successfully")

    def clean(self) -> None:
        """clean credentials from disk"""

        logger.debug("removing credentials")

        if self.secrets_file.exists():
            self.secrets_file.unlink()

        logger.info("credentials removed")



from enum import Enum


class Keys(Enum):
    USERNAME = "XLEAP_USERNAME"
    API_KEY = "XLEAP_API_KEY"
    PROJECT_ID = "XLEAP_PROJECT_ID"
    API_ENDPOINT = "XLEAP_API_ENDPOINT"

    @property
    def value(self) -> str:
        return f"XLEAP_{self.name}"

    @property
    def suffix(self):
        return self.name.lower()

    def __eq__(self, __o: object) -> bool:
        return self.value == __o



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.data_point_result_type_0 import DataPointResultType0


T = TypeVar("T", bound="DataPoint")


@_attrs_define
class DataPoint:
    """
    Attributes:
        id (Union[Unset, str]):
        question (Union[None, Unset, str]):
        answer (Union[None, Unset, str]):
        contexts (Union[Unset, List[str]]):
        ground_truths (Union[Unset, List[str]]):
        result (Union['DataPointResultType0', None, Unset]):
        project (Union[None, Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    question: Union[None, Unset, str] = UNSET
    answer: Union[None, Unset, str] = UNSET
    contexts: Union[Unset, List[str]] = UNSET
    ground_truths: Union[Unset, List[str]] = UNSET
    result: Union["DataPointResultType0", None, Unset] = UNSET
    project: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.data_point_result_type_0 import DataPointResultType0

        id = self.id

        question: Union[None, Unset, str]
        if isinstance(self.question, Unset):
            question = UNSET
        else:
            question = self.question

        answer: Union[None, Unset, str]
        if isinstance(self.answer, Unset):
            answer = UNSET
        else:
            answer = self.answer

        contexts: Union[Unset, List[str]] = UNSET
        if not isinstance(self.contexts, Unset):
            contexts = self.contexts

        ground_truths: Union[Unset, List[str]] = UNSET
        if not isinstance(self.ground_truths, Unset):
            ground_truths = self.ground_truths

        result: Union[Dict[str, Any], None, Unset]
        if isinstance(self.result, Unset):
            result = UNSET
        elif isinstance(self.result, DataPointResultType0):
            result = self.result.to_dict()
        else:
            result = self.result

        project: Union[None, Unset, str]
        if isinstance(self.project, Unset):
            project = UNSET
        else:
            project = self.project

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if question is not UNSET:
            field_dict["question"] = question
        if answer is not UNSET:
            field_dict["answer"] = answer
        if contexts is not UNSET:
            field_dict["contexts"] = contexts
        if ground_truths is not UNSET:
            field_dict["ground_truths"] = ground_truths
        if result is not UNSET:
            field_dict["result"] = result
        if project is not UNSET:
            field_dict["project"] = project

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.data_point_result_type_0 import DataPointResultType0

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        def _parse_question(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        question = _parse_question(d.pop("question", UNSET))

        def _parse_answer(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        answer = _parse_answer(d.pop("answer", UNSET))

        contexts = cast(List[str], d.pop("contexts", UNSET))

        ground_truths = cast(List[str], d.pop("ground_truths", UNSET))

        def _parse_result(data: object) -> Union["DataPointResultType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                result_type_0 = DataPointResultType0.from_dict(data)

                return result_type_0
            except:  # noqa: E722
                pass
            return cast(Union["DataPointResultType0", None, Unset], data)

        result = _parse_result(d.pop("result", UNSET))

        def _parse_project(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        project = _parse_project(d.pop("project", UNSET))

        data_point = cls(
            id=id,
            question=question,
            answer=answer,
            contexts=contexts,
            ground_truths=ground_truths,
            result=result,
            project=project,
        )

        data_point.additional_properties = d
        return data_point

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .data_point import DataPoint
from .data_point_result_type_0 import DataPointResultType0
from .list_metrics_datas_response_200 import ListMetricsDatasResponse200
from .list_ml_models_response_200 import ListMLModelsResponse200
from .list_org_lists_response_200 import ListOrgListsResponse200
from .list_project_lists_response_200 import ListProjectListsResponse200
from .list_prompts_response_200 import ListPromptsResponse200
from .list_rag_data_points_response_200 import ListRagDataPointsResponse200
from .list_runs_response_200 import ListRunsResponse200
from .metrics import Metrics
from .ml_model import MLModel
from .ml_model_status import MLModelStatus
from .org_detail import OrgDetail
from .org_detail_members_item import OrgDetailMembersItem
from .org_list import OrgList
from .project_detail import ProjectDetail
from .project_list import ProjectList
from .prompt import Prompt
from .run import Run
from .run_create import RunCreate
from .run_create_events_type_0 import RunCreateEventsType0
from .run_create_extra_type_0 import RunCreateExtraType0
from .run_create_inputs_type_0 import RunCreateInputsType0
from .run_create_outputs_type_0 import RunCreateOutputsType0
from .run_create_serialized_type_0 import RunCreateSerializedType0
from .run_events_type_0 import RunEventsType0
from .run_extra_type_0 import RunExtraType0
from .run_inputs_type_0 import RunInputsType0
from .run_outputs_type_0 import RunOutputsType0
from .run_serialized_type_0 import RunSerializedType0

__all__ = (
    "DataPoint",
    "DataPointResultType0",
    "ListMetricsDatasResponse200",
    "ListMLModelsResponse200",
    "ListOrgListsResponse200",
    "ListProjectListsResponse200",
    "ListPromptsResponse200",
    "ListRagDataPointsResponse200",
    "ListRunsResponse200",
    "Metrics",
    "MLModel",
    "MLModelStatus",
    "OrgDetail",
    "OrgDetailMembersItem",
    "OrgList",
    "ProjectDetail",
    "ProjectList",
    "Prompt",
    "Run",
    "RunCreate",
    "RunCreateEventsType0",
    "RunCreateExtraType0",
    "RunCreateInputsType0",
    "RunCreateOutputsType0",
    "RunCreateSerializedType0",
    "RunEventsType0",
    "RunExtraType0",
    "RunInputsType0",
    "RunOutputsType0",
    "RunSerializedType0",
)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.ml_model_status import MLModelStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="MLModel")


@_attrs_define
class MLModel:
    """
    Attributes:
        name (str):
        url (str):
        requests_count (int):
        cost (str):
        status (MLModelStatus):
        id (Union[Unset, str]):
    """

    name: str
    url: str
    requests_count: int
    cost: str
    status: MLModelStatus
    id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        url = self.url

        requests_count = self.requests_count

        cost = self.cost

        status = self.status.value

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "url": url,
                "requests_count": requests_count,
                "cost": cost,
                "status": status,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        url = d.pop("url")

        requests_count = d.pop("requests_count")

        cost = d.pop("cost")

        status = MLModelStatus(d.pop("status"))

        id = d.pop("id", UNSET)

        ml_model = cls(
            name=name,
            url=url,
            requests_count=requests_count,
            cost=cost,
            status=status,
            id=id,
        )

        ml_model.additional_properties = d
        return ml_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import logging
import os
from functools import partial

import requests

from xleap.client.api.data import (
    create_rag_data_point,
    destroy_rag_data_point,
    list_rag_data_points,
    retrieve_rag_data_point,
    update_rag_data_point,
)
from xleap.config import Keys
from xleap.utils.login import Auth

logger = logging.getLogger(__name__)


from xleap.client import Client


class Xleap:
    session = requests.Session()

    def __init__(
        self, api_key: str = None, base_url: str = "http://localhost:8000"
    ) -> None:
        self.__api_key = (
            os.getenv(Keys.API_KEY.value) or api_key or self._load_auth_header()
        )
        if not self.__api_key:
            logger.warning("api key not available")

        self.base_url = os.getenv(Keys.API_ENDPOINT.value) or base_url

        self.client = Client(
            self.base_url,
            headers={
                "content-type": "application/json",
                "Authorization": self.__api_key,
            },
        )

    def _load_auth_header(self):
        auth = Auth()
        auth.from_env()
        return auth.auth_header

    @property
    def data(self):
        class Data:
            create = partial(
                create_rag_data_point.sync_detailed, client=self.client
            )
            destroy = partial(
                destroy_rag_data_point.sync_detailed, client=self.client
            )
            retrieve = partial(
                retrieve_rag_data_point.sync_detailed, client=self.client
            )
            update = partial(
                update_rag_data_point.sync_detailed, client=self.client
            )
            list = partial(
                list_rag_data_points.sync_detailed, client=self.client
            )

        return Data()




from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.run_events_type_0 import RunEventsType0
    from ..models.run_extra_type_0 import RunExtraType0
    from ..models.run_inputs_type_0 import RunInputsType0
    from ..models.run_outputs_type_0 import RunOutputsType0
    from ..models.run_serialized_type_0 import RunSerializedType0


T = TypeVar("T", bound="Run")


@_attrs_define
class Run:
    """
    Attributes:
        answer (str):
        start_time (str):
        id (Union[Unset, str]):
        session_name (Union[None, Unset, str]):
        name (Union[None, Unset, str]):
        reference_example_id (Union[None, Unset, str]):
        end_time (Union[None, Unset, str]):
        extra (Union['RunExtraType0', None, Unset]):
        events (Union['RunEventsType0', None, Unset]):
        outputs (Union['RunOutputsType0', None, Unset]):
        serialized (Union['RunSerializedType0', None, Unset]):
        inputs (Union['RunInputsType0', None, Unset]):
        error (Union[None, Unset, str]):
        run_type (Union[None, Unset, str]):
        execution_order (Union[None, Unset, int]):
        child_execution_order (Union[None, Unset, str]):
        parent_run_id (Union[None, Unset, str]):
    """

    answer: str
    start_time: str
    id: Union[Unset, str] = UNSET
    session_name: Union[None, Unset, str] = UNSET
    name: Union[None, Unset, str] = UNSET
    reference_example_id: Union[None, Unset, str] = UNSET
    end_time: Union[None, Unset, str] = UNSET
    extra: Union["RunExtraType0", None, Unset] = UNSET
    events: Union["RunEventsType0", None, Unset] = UNSET
    outputs: Union["RunOutputsType0", None, Unset] = UNSET
    serialized: Union["RunSerializedType0", None, Unset] = UNSET
    inputs: Union["RunInputsType0", None, Unset] = UNSET
    error: Union[None, Unset, str] = UNSET
    run_type: Union[None, Unset, str] = UNSET
    execution_order: Union[None, Unset, int] = UNSET
    child_execution_order: Union[None, Unset, str] = UNSET
    parent_run_id: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.run_events_type_0 import RunEventsType0
        from ..models.run_extra_type_0 import RunExtraType0
        from ..models.run_inputs_type_0 import RunInputsType0
        from ..models.run_outputs_type_0 import RunOutputsType0
        from ..models.run_serialized_type_0 import RunSerializedType0

        answer = self.answer

        start_time = self.start_time

        id = self.id

        session_name: Union[None, Unset, str]
        if isinstance(self.session_name, Unset):
            session_name = UNSET
        else:
            session_name = self.session_name

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        reference_example_id: Union[None, Unset, str]
        if isinstance(self.reference_example_id, Unset):
            reference_example_id = UNSET
        else:
            reference_example_id = self.reference_example_id

        end_time: Union[None, Unset, str]
        if isinstance(self.end_time, Unset):
            end_time = UNSET
        else:
            end_time = self.end_time

        extra: Union[Dict[str, Any], None, Unset]
        if isinstance(self.extra, Unset):
            extra = UNSET
        elif isinstance(self.extra, RunExtraType0):
            extra = self.extra.to_dict()
        else:
            extra = self.extra

        events: Union[Dict[str, Any], None, Unset]
        if isinstance(self.events, Unset):
            events = UNSET
        elif isinstance(self.events, RunEventsType0):
            events = self.events.to_dict()
        else:
            events = self.events

        outputs: Union[Dict[str, Any], None, Unset]
        if isinstance(self.outputs, Unset):
            outputs = UNSET
        elif isinstance(self.outputs, RunOutputsType0):
            outputs = self.outputs.to_dict()
        else:
            outputs = self.outputs

        serialized: Union[Dict[str, Any], None, Unset]
        if isinstance(self.serialized, Unset):
            serialized = UNSET
        elif isinstance(self.serialized, RunSerializedType0):
            serialized = self.serialized.to_dict()
        else:
            serialized = self.serialized

        inputs: Union[Dict[str, Any], None, Unset]
        if isinstance(self.inputs, Unset):
            inputs = UNSET
        elif isinstance(self.inputs, RunInputsType0):
            inputs = self.inputs.to_dict()
        else:
            inputs = self.inputs

        error: Union[None, Unset, str]
        if isinstance(self.error, Unset):
            error = UNSET
        else:
            error = self.error

        run_type: Union[None, Unset, str]
        if isinstance(self.run_type, Unset):
            run_type = UNSET
        else:
            run_type = self.run_type

        execution_order: Union[None, Unset, int]
        if isinstance(self.execution_order, Unset):
            execution_order = UNSET
        else:
            execution_order = self.execution_order

        child_execution_order: Union[None, Unset, str]
        if isinstance(self.child_execution_order, Unset):
            child_execution_order = UNSET
        else:
            child_execution_order = self.child_execution_order

        parent_run_id: Union[None, Unset, str]
        if isinstance(self.parent_run_id, Unset):
            parent_run_id = UNSET
        else:
            parent_run_id = self.parent_run_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "answer": answer,
                "start_time": start_time,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if session_name is not UNSET:
            field_dict["session_name"] = session_name
        if name is not UNSET:
            field_dict["name"] = name
        if reference_example_id is not UNSET:
            field_dict["reference_example_id"] = reference_example_id
        if end_time is not UNSET:
            field_dict["end_time"] = end_time
        if extra is not UNSET:
            field_dict["extra"] = extra
        if events is not UNSET:
            field_dict["events"] = events
        if outputs is not UNSET:
            field_dict["outputs"] = outputs
        if serialized is not UNSET:
            field_dict["serialized"] = serialized
        if inputs is not UNSET:
            field_dict["inputs"] = inputs
        if error is not UNSET:
            field_dict["error"] = error
        if run_type is not UNSET:
            field_dict["run_type"] = run_type
        if execution_order is not UNSET:
            field_dict["execution_order"] = execution_order
        if child_execution_order is not UNSET:
            field_dict["child_execution_order"] = child_execution_order
        if parent_run_id is not UNSET:
            field_dict["parent_run_id"] = parent_run_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.run_events_type_0 import RunEventsType0
        from ..models.run_extra_type_0 import RunExtraType0
        from ..models.run_inputs_type_0 import RunInputsType0
        from ..models.run_outputs_type_0 import RunOutputsType0
        from ..models.run_serialized_type_0 import RunSerializedType0

        d = src_dict.copy()
        answer = d.pop("answer")

        start_time = d.pop("start_time")

        id = d.pop("id", UNSET)

        def _parse_session_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        session_name = _parse_session_name(d.pop("session_name", UNSET))

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_reference_example_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        reference_example_id = _parse_reference_example_id(
            d.pop("reference_example_id", UNSET)
        )

        def _parse_end_time(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        end_time = _parse_end_time(d.pop("end_time", UNSET))

        def _parse_extra(data: object) -> Union["RunExtraType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                extra_type_0 = RunExtraType0.from_dict(data)

                return extra_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunExtraType0", None, Unset], data)

        extra = _parse_extra(d.pop("extra", UNSET))

        def _parse_events(data: object) -> Union["RunEventsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                events_type_0 = RunEventsType0.from_dict(data)

                return events_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunEventsType0", None, Unset], data)

        events = _parse_events(d.pop("events", UNSET))

        def _parse_outputs(data: object) -> Union["RunOutputsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                outputs_type_0 = RunOutputsType0.from_dict(data)

                return outputs_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunOutputsType0", None, Unset], data)

        outputs = _parse_outputs(d.pop("outputs", UNSET))

        def _parse_serialized(data: object) -> Union["RunSerializedType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                serialized_type_0 = RunSerializedType0.from_dict(data)

                return serialized_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunSerializedType0", None, Unset], data)

        serialized = _parse_serialized(d.pop("serialized", UNSET))

        def _parse_inputs(data: object) -> Union["RunInputsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                inputs_type_0 = RunInputsType0.from_dict(data)

                return inputs_type_0
            except:  # noqa: E722
                pass
            return cast(Union["RunInputsType0", None, Unset], data)

        inputs = _parse_inputs(d.pop("inputs", UNSET))

        def _parse_error(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error = _parse_error(d.pop("error", UNSET))

        def _parse_run_type(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        run_type = _parse_run_type(d.pop("run_type", UNSET))

        def _parse_execution_order(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        execution_order = _parse_execution_order(d.pop("execution_order", UNSET))

        def _parse_child_execution_order(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        child_execution_order = _parse_child_execution_order(
            d.pop("child_execution_order", UNSET)
        )

        def _parse_parent_run_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        parent_run_id = _parse_parent_run_id(d.pop("parent_run_id", UNSET))

        run = cls(
            answer=answer,
            start_time=start_time,
            id=id,
            session_name=session_name,
            name=name,
            reference_example_id=reference_example_id,
            end_time=end_time,
            extra=extra,
            events=events,
            outputs=outputs,
            serialized=serialized,
            inputs=inputs,
            error=error,
            run_type=run_type,
            execution_order=execution_order,
            child_execution_order=child_execution_order,
            parent_run_id=parent_run_id,
        )

        run.additional_properties = d
        return run

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="OrgDetailMembersItem")


@_attrs_define
class OrgDetailMembersItem:
    """
    Attributes:
        username (str): Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.
        id (Union[Unset, int]):
        first_name (Union[Unset, str]):
        last_name (Union[Unset, str]):
        email (Union[Unset, str]):
    """

    username: str
    id: Union[Unset, int] = UNSET
    first_name: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        username = self.username

        id = self.id

        first_name = self.first_name

        last_name = self.last_name

        email = self.email

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "username": username,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if first_name is not UNSET:
            field_dict["first_name"] = first_name
        if last_name is not UNSET:
            field_dict["last_name"] = last_name
        if email is not UNSET:
            field_dict["email"] = email

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        username = d.pop("username")

        id = d.pop("id", UNSET)

        first_name = d.pop("first_name", UNSET)

        last_name = d.pop("last_name", UNSET)

        email = d.pop("email", UNSET)

        org_detail_members_item = cls(
            username=username,
            id=id,
            first_name=first_name,
            last_name=last_name,
            email=email,
        )

        org_detail_members_item.additional_properties = d
        return org_detail_members_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
